/* 這個檔案沒有版權, 可以隨意修改. */
/* 很好的參考點: http://www.westciv.com/style_master/academy/css_tutorial/ */
/* 最簡單語法:  ELEMENT_SELECTOR.CLASS_SELECTOR { PROPERTY: VALUE; } */

/* body { font-family: "AR PL KaitiM Big5" } */

em, strong {
    text-decoration: underline;
    font-style: normal;
/*
    font-weight: bold;
*/
}

pre, code, .code { font-family: courier, monospace; }
code, .code { font-weight: bold; }

code em { text-decoration: none; }

table.center-content td { text-align: center; }
th {
    font-family: helvetica;
    font-weight: bold;
    text-align: left;
    vertical-align: top;
}

.kbd {
    margin-left: 0.1em;
    margin-right: 0.1em;
}

code.lit { font-weight: bold; }
code.id { text-decoration: underline; }
ul.empty { list-style: none; }
h1 {
    font-size: 150%;
    text-align: center;
    counter-reset: h2;
}
h2 {
    font-size: 120%;
    text-align: center; 
    text-decoration: underline;
    counter-increment: h2;
    /* see http://www.davidflanagan.com/blog/2005_08.html#000075 */
}
h3 { counter-increment: h3; }
/* h2:before { counter-reset: h3; } */
h2.sec:before {
    content: "第" counter(h2) "節 ";
}
h2.sec-en:before {
    content: "Section " counter(h2) ": ";
}
h3.sec:before {
    content: counter(h2) "." counter(h3) " ";
}
p { margin: 1em; text-indent: 2em; }
p.publication { text-indent: 0; text-align: center; }
.center { text-align: center; }
.small { font-size: small; }
p.footer { font-size: small; }
p.noindent { text-indent: 0; }
ol.upper-latin { list-style-type: upper-latin; }

img.left { float: left; }
img.right { float: right; }
img.alone { clear: both; }

/*
div {
    position: absolute;
    overflow: auto;
}
*/

div#toc {
    position: fixed; /* 不管頁面 scroll 到那裡, 這個 div 永遠釘在此處 */
    /* _position: absolute; 感謝 http://vbb3.twftp.org/ 的 Crocodile
       提供; 但這會令本檔失去 w3c valid css 的認證 ... */
    top: 0; bottom: 0;
    right: 0; width: 7em; 
    /* border-right: 0.2em solid #ff00ff; */
}

/* 「滑鼠移到 link 上, 就跳出說明文字」 的特效是從這裡學來的:
   http://www.meyerweb.com/eric/css/edge/popups/demo.html */
div#toc a .hovertext { display: none; }
div#toc a:hover .hovertext {
    display: block;
    border: 2pt solid red;
    /* padding-left: 0.5em; padding-right: 0.5em; */
    position: absolute;
    top: 1em;
    /* text-align: center; */
    font-size: 80%;
    line-height: 100%;
}

div#content {
    position: absolute; /* 右邊要留空間給 div#toc, 以免被它蓋住 */
    top: 2em; bottom: 0;
    left: 0; right: 7em;
}

@media screen {
    body {
	margin-left: 0%; margin-right: 0%;
	margin-top: 0%; margin-bottom: 0%;
	line-height: 140%;
	font-size: 14pt;
    }
}


@media print {
/* 試半天的結論: 在 body 內, 左右 padding 與 左右 margin
   都可以達到留邊的效果; 至於 @page 不論放在 @media 之內或之外,
   都完全沒有效果 (連 border 都沒有作用) */
/*
@page { size: 8.5in 11in; margin: 5cm; }
@page { size: auto; margin: 5cm; }
@page { border: 2pt solid red; }
*/
    body {
/*	border: 1pt solid black; */
/*	padding-left: 5cm; padding-right: 1cm; */
	margin-left: 0cm; margin-right: 0cm;
	font-size: 12pt;
	line-height: 140%;
    }
    .noprint { display: none; }
    div#toc { display: none; }
    div#content { right: 0em; position: static; }
    /* 如果沒有 "position: static;" 則只會印出第一頁 */
    dfn, code { margin-left: 0.1em; margin-right: 0.6em; color: black; }
    em { margin-left: 0.2em; margin-right: 0.3em; }
    a { text-decoration: none; margin-left: 0.6em; margin-right: 1.0em; }
}

